@@ -33,7 +33,7 @@
 #endif
 
 #include <glib/gi18n.h>
-#include <glib/gfileutils.h>
+#include <glib.h>
 #include <gio/gio.h>
 #include <string.h>
 
@@ -114,7 +114,7 @@ async_data_new (GeditGioDocumentSaver *gvsaver)
 {
 	AsyncData *async;
 	
-	async = g_new (AsyncData, 1);
+	async = g_slice_new (AsyncData);
 	async->saver = gvsaver;
 	async->buffer = NULL;
 	async->cancellable = g_object_ref (gvsaver->priv->cancellable);
@@ -128,7 +128,7 @@ async_data_free (AsyncData *async)
 {
 	g_object_unref (async->cancellable);
 	g_free (async->buffer);
-	g_free (async);
+	g_slice_free (AsyncData, async);
 }
 
 static void 
@@ -394,9 +394,9 @@ async_write_cb (GOutputStream *stream,
 
 	gvsaver = async->saver;
 	bytes_written = g_output_stream_write_finish (stream, res, &error);
-	
-	gedit_debug_message (DEBUG_SAVER, "Written: %d", bytes_written);
-	
+
+	gedit_debug_message (DEBUG_SAVER, "Written: %" G_GSSIZE_FORMAT, bytes_written);
+
 	if (bytes_written == -1)
 	{
 		gedit_debug_message (DEBUG_SAVER, "Write error: %s", error->message);
@@ -432,7 +432,9 @@ write_file_chunk (AsyncData *async)
 
 	gvsaver = async->saver;
 
-	gedit_debug_message (DEBUG_SAVER, "Writing next chunk: %Ld/%Ld", gvsaver->priv->bytes_written, gvsaver->priv->size);
+	gedit_debug_message (DEBUG_SAVER,
+			     "Writing next chunk: %" G_GINT64_FORMAT "/%" G_GINT64_FORMAT,
+			     gvsaver->priv->bytes_written, gvsaver->priv->size);
 
 	g_output_stream_write_async (G_OUTPUT_STREAM (gvsaver->priv->stream),
 				     async->buffer + gvsaver->priv->bytes_written,
@@ -547,10 +549,10 @@ begin_write (AsyncData *async)
 
 	async->buffer = buffer;
 	gvsaver->priv->size = len;
-	
-	gedit_debug_message (DEBUG_SAVER, "File contents size: %Ld", gvsaver->priv->size);
+
+	gedit_debug_message (DEBUG_SAVER, "File contents size: %" G_GINT64_FORMAT, gvsaver->priv->size);
 	gedit_debug_message (DEBUG_SAVER, "Calling replace_async");
-	
+
 	g_file_replace_async (gvsaver->priv->gfile, 
 			      NULL,
 			      FALSE,
@@ -617,7 +619,7 @@ check_modification_callback (GFile        *source,
 			     AsyncData    *async)
 {
 	GeditGioDocumentSaver *gvsaver;
-	GError *error;
+	GError *error = NULL;
 	GFileInfo *info;
 	
 	/* manually check cancelled state */
@@ -638,14 +640,18 @@ check_modification_callback (GFile        *source,
 			return;
 		}
 		
-		gedit_debug_message (DEBUG_SAVER, "Error getting modification: %s", error->message);
+		/* it's perfectly fine if the file doesn't exist yet */
+		if (error->code != G_IO_ERROR_NOT_FOUND)
+		{
+			gedit_debug_message (DEBUG_SAVER, "Error getting modification: %s", error->message);
 
-		async_failed (async, error);
-		return;
+			async_failed (async, error);
+			return;
+		}
 	}
 
 	/* check if the mtime is > what we know about it (if we have it) */
-	if (g_file_info_has_attribute (info,
+	if (info != NULL && g_file_info_has_attribute (info,
 				       G_FILE_ATTRIBUTE_TIME_MODIFIED))
 	{
 		GTimeVal mtime;
@@ -669,7 +675,8 @@ check_modification_callback (GFile        *source,
 		}
 	}
 
-	g_object_unref (info);
+	if (info != NULL)
+		g_object_unref (info);
 
 	/* modification check passed, start write */
 	begin_write (async);
