@@ -152,7 +152,12 @@ add_file (GeditFileBookmarksStore *model,
 		pixbuf = gedit_file_browser_utils_pixbuf_from_theme ("drive-harddisk", GTK_ICON_SIZE_MENU);
 
 	if (pixbuf == NULL) {
-		pixbuf = gedit_file_browser_utils_pixbuf_from_file (file, GTK_ICON_SIZE_MENU);
+		/* getting the icon is a sync get_info call, so we just do it for local files */
+		if (native) {
+			pixbuf = gedit_file_browser_utils_pixbuf_from_file (file, GTK_ICON_SIZE_MENU);
+		} else {
+			pixbuf = gedit_file_browser_utils_pixbuf_from_theme ("folder", GTK_ICON_SIZE_MENU);
+		}
 	}
 
 	if (name == NULL) {
@@ -468,7 +473,6 @@ add_bookmark (GeditFileBookmarksStore * model,
 	gboolean ret;
 	guint flags = GEDIT_FILE_BOOKMARKS_STORE_IS_BOOKMARK;
 	GtkTreeIter iter;
-	GdkPixbuf * pixbuf;
 	
 	file = g_file_new_for_uri (uri);
 	
@@ -479,32 +483,7 @@ add_bookmark (GeditFileBookmarksStore * model,
 	}
 
 	ret = add_file (model, file, name, flags, &iter);
-	
-	if (!g_file_is_native (file)) {
-		/* Check icon */
-		gtk_tree_model_get (GTK_TREE_MODEL (model), 
-				    &iter, 
-				    GEDIT_FILE_BOOKMARKS_STORE_COLUMN_ICON, 
-				    &pixbuf, 
-				    -1);
 
-		if (!pixbuf) {
-			pixbuf = gedit_file_browser_utils_pixbuf_from_theme ("folder", GTK_ICON_SIZE_MENU);
-
-			if (pixbuf) {			
-				gtk_tree_store_set (GTK_TREE_STORE (model), 
-						    &iter, 
-						    GEDIT_FILE_BOOKMARKS_STORE_COLUMN_ICON,
-						    pixbuf,
-						    -1);
-						    
-				g_object_unref (pixbuf);
-			}
-		} else {
-			g_object_unref (pixbuf);
-		}
-	}
-	
 	g_object_unref (file);
 	
 	return ret;
@@ -617,15 +596,23 @@ bookmarks_compare_names (GtkTreeModel * model, GtkTreeIter * a,
 	gchar *n1;
 	gchar *n2;
 	gint result;
+	guint f1;
+	guint f2;
 
 	gtk_tree_model_get (model, a,
 			    GEDIT_FILE_BOOKMARKS_STORE_COLUMN_NAME, &n1,
+			    GEDIT_FILE_BOOKMARKS_STORE_COLUMN_FLAGS, &f1,
 			    -1);
 	gtk_tree_model_get (model, b,
 			    GEDIT_FILE_BOOKMARKS_STORE_COLUMN_NAME, &n2,
+			    GEDIT_FILE_BOOKMARKS_STORE_COLUMN_FLAGS, &f2,
 			    -1);
 
-	if (n1 == NULL && n2 == NULL)
+	/* do not sort actual bookmarks to keep same order as in nautilus */
+	if ((f1 & GEDIT_FILE_BOOKMARKS_STORE_IS_BOOKMARK) &&
+	    (f2 & GEDIT_FILE_BOOKMARKS_STORE_IS_BOOKMARK))
+		result = 0;
+	else if (n1 == NULL && n2 == NULL)
 		result = 0;
 	else if (n1 == NULL)
 		result = -1;
